@@ -18,20 +18,6 @@ import logging
 logging.basicConfig()
 
 
-def proxy_get_object_path(proxy):
-    if getattr(dbus, 'version', (0,0,0)) < (0, 80):
-        # non-public API in dbus-python < 0.80
-        return proxy._object_path
-    else:
-        # actually official API in 0.80+
-        return proxy.__dbus_object_path__
-
-
-def proxy_get_service_name(proxy):
-    # FIXME: using non-public API
-    return proxy._named_service
-
-
 def get_stream_engine():
     bus = dbus.Bus()
     return bus.get_object(
@@ -39,16 +25,20 @@ def get_stream_engine():
         '/org/freedesktop/Telepathy/StreamEngine')
 
 class Call:
-    def __init__(self, conn):
-        self.conn = conn
+    def __init__(self, account_file):
+        self.conn = connection_from_file(account_file,
+            ready_handler=self.ready_cb)
         self.channel = None
 
-        conn[CONN_INTERFACE].connect_to_signal('StatusChanged',
+        self.conn[CONN_INTERFACE].connect_to_signal('StatusChanged',
             self.status_changed_cb)
-        conn[CONN_INTERFACE].connect_to_signal('NewChannel',
+        self.conn[CONN_INTERFACE].connect_to_signal('NewChannel',
             self.new_channel_cb)
 
     def run(self):
+        print "connecting"
+        self.conn[CONN_INTERFACE].Connect()
+
         self.loop = gobject.MainLoop()
 
         try:
@@ -60,6 +50,12 @@ class Call:
                 print "closing channel"
                 self.channel[CHANNEL_INTERFACE].Close()
 
+        try:
+            print "disconnecting"
+            self.conn[CONN_INTERFACE].Disconnect()
+        except dbus.DBusException:
+            pass
+
     def quit(self):
         if self.loop:
             self.loop.quit()
@@ -70,6 +66,9 @@ class Call:
             print 'connection closed'
             self.quit()
 
+    def ready_cb(self, conn):
+        pass
+
     def request_channel_error_cb(self, exception):
         print 'error:', exception
         self.quit()
@@ -83,7 +82,7 @@ class Call:
         self.chan_handle = handle
 
         print "new streamed media channel"
-        Channel(proxy_get_service_name(self.conn._dbus_object), object_path,
+        Channel(self.conn.service_name, object_path,
                 ready_handler=self.channel_ready_cb)
 
     def channel_ready_cb(self, channel):
@@ -96,10 +95,10 @@ class Call:
         handler = dbus.Interface(stream_engine,
             'org.freedesktop.Telepathy.ChannelHandler')
         handler.HandleChannel(
-            proxy_get_service_name(self.conn._dbus_object),
-            proxy_get_object_path(self.conn._dbus_object),
+            self.conn.service_name,
+            self.conn.object_path,
             CHANNEL_TYPE_STREAMED_MEDIA,
-            proxy_get_object_path(channel._dbus_object),
+            channel.object_path,
             self.chan_handle_type,
             self.chan_handle)
 
@@ -115,29 +114,26 @@ class Call:
             added, removed, local_pending, remote_pending, actor, reason)
 
 class OutgoingCall(Call):
-    def __init__(self, conn, contact):
-        Call.__init__(self, conn)
+    def __init__(self, account_file, contact):
+        Call.__init__(self, account_file)
         self.contact = contact
 
-    def status_changed_cb (self, state, reason):
-        if state == CONNECTION_STATUS_CONNECTED:
-            handle = self.conn[CONN_INTERFACE].RequestHandles(
-                CONNECTION_HANDLE_TYPE_CONTACT, [self.contact])[0]
-            self.handle = handle
+    def ready_cb(self, conn):
+        handle = self.conn[CONN_INTERFACE].RequestHandles(
+            CONNECTION_HANDLE_TYPE_CONTACT, [self.contact])[0]
+        self.handle = handle
 
-            print 'got handle %d for %s' % (handle, self.contact)
+        print 'got handle %d for %s' % (handle, self.contact)
 
-            # hack
-            import time
-            time.sleep(5)
+        # hack - wait for capabilities to come in
+        import time
+        time.sleep(5)
 
-            self.conn[CONN_INTERFACE].RequestChannel(
-                CHANNEL_TYPE_STREAMED_MEDIA, CONNECTION_HANDLE_TYPE_NONE,
-                0, True,
-                reply_handler=lambda *stuff: None,
-                error_handler=self.request_channel_error_cb)
-
-        Call.status_changed_cb(self, state, reason)
+        self.conn[CONN_INTERFACE].RequestChannel(
+            CHANNEL_TYPE_STREAMED_MEDIA, CONNECTION_HANDLE_TYPE_NONE,
+            0, True,
+            reply_handler=lambda *stuff: None,
+            error_handler=self.request_channel_error_cb)
 
     def channel_ready_cb(self, channel):
         Call.channel_ready_cb(self, channel)
@@ -163,15 +159,9 @@ class OutgoingCall(Call):
                 self.quit()
 
 class IncomingCall(Call):
-    def status_changed_cb (self, state, reason):
-        if state == CONNECTION_STATUS_CONNECTED:
-            print "waiting for incoming call"
-
-            self.conn._valid_interfaces.add(CONN_INTERFACE_CAPABILITIES)
-            self.conn[CONN_INTERFACE_CAPABILITIES].AdvertiseCapabilities(
-                [(CHANNEL_TYPE_STREAMED_MEDIA, 3)], [])
-
-        Call.status_changed_cb(self, state, reason)
+    def ready_cb(self, conn):
+        self.conn[CONN_INTERFACE_CAPABILITIES].AdvertiseCapabilities(
+            [(CHANNEL_TYPE_STREAMED_MEDIA, 3)], [])
 
     def channel_ready_cb(self, channel):
         Call.channel_ready_cb(self, channel)
@@ -181,22 +171,14 @@ class IncomingCall(Call):
         channel[CHANNEL_INTERFACE_GROUP].AddMembers(pending, "")
 
 if __name__ == '__main__':
-    assert len(sys.argv) in (2, 3)
-    conn = connection_from_file(sys.argv[1])
+    args = sys.argv[1:]
 
-    if len(sys.argv) > 2:
-        contact = sys.argv[2]
-        call = OutgoingCall(conn, sys.argv[2])
+    assert len(args) in (1, 2)
+
+    if len(args) > 1:
+        contact = args[1]
+        call = OutgoingCall(args[0], args[1])
     else:
-        call = IncomingCall(conn)
+        call = IncomingCall(args[0])
 
-    print "connecting"
-    conn[CONN_INTERFACE].Connect()
     call.run()
-
-    try:
-        print "disconnecting"
-        conn[CONN_INTERFACE].Disconnect()
-    except dbus.DBusException:
-        pass
-
